/*
 * Author: yeras-is
 */

import 'package:flutter/cupertino.dart';
import 'dart:math';

import 'package:flutter/material.dart';

class RotatedCard extends StatefulWidget {
  /// Widget on first card
  final Widget firstCard;

  /// Widget on second card
  final Widget secondCard;

  /// Duration of rotate
  final Duration duration;

  const RotatedCard({
    Key key,
    @required this.firstCard,
    @required this.secondCard,
    this.duration,
  }) : super(key: key);

  @override
  _RotatedCardState createState() => _RotatedCardState();
}

class _RotatedCardState extends State<RotatedCard> with SingleTickerProviderStateMixin {
  AnimationController _animationController;
  Animation _animation;
  AnimationStatus _animationStatus = AnimationStatus.dismissed;

  @override
  void initState() {
    super.initState();
    _animationController =
        AnimationController(vsync: this, duration: widget.duration ?? Duration(milliseconds: 500));
    _animation = Tween(end: 1.0, begin: 0.0).animate(_animationController)
      ..addListener(() {
        setState(() {});
      })
      ..addStatusListener((status) {
        _animationStatus = status;
      });
  }

  @override
  void dispose() {
    _animationController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        if (_animationStatus == AnimationStatus.dismissed) {
          _animationController.forward();
        } else {
          _animationController.reverse();
        }
      },
      child: Transform(
        alignment: FractionalOffset.center,
        transform: Matrix4.identity()
          ..setEntry(3, 2, 0.0025)
          ..rotateY((pi * _animation.value)),
        child: _animation.value <= 0.5
            ? widget.firstCard ??
                Container(
                  width: 200,
                  height: 300,
                  color: Colors.blueAccent,
                  child: Icon(Icons.report_problem),
                )
            : Transform(
                alignment: FractionalOffset.center,
                transform: Matrix4.identity()
                  ..setEntry(3, 2, 0.001)
                  ..rotateY(1 * 3.14),
                child: widget.secondCard ??
                    Container(
                      width: 200,
                      height: 300,
                      color: Colors.redAccent,
                      child: Icon(Icons.description),
                    ),
              ),
      ),
    );
  }
}
