/*
 * Author: yeras-is
 */
import 'package:flutter/cupertino.dart';
import 'package:flutter/widgets.dart';

class TwoViewListBuilder<T> extends StatelessWidget {
  /// set view type
  final bool isListView;

  /// entity to Widget mapper for List
  final Function(T model) listMap;

  ///  List of Data
  final List json;

  /// entity to Widget mapper for Wrap
  final Function(T model) blockMap;

  /// duration of animation
  final Duration duration;
  final Curve curve;

  const TwoViewListBuilder({
    Key key,
    this.isListView,
    this.listMap,
    this.json,
    this.blockMap,
    this.duration,
    this.curve,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      physics: BouncingScrollPhysics(),
      child: AnimatedCrossFade(
        duration: duration ?? Duration(milliseconds: 300),
        firstCurve: curve ?? Curves.fastLinearToSlowEaseIn,
        secondCurve: curve ?? Curves.fastLinearToSlowEaseIn,
        crossFadeState: isListView ? CrossFadeState.showFirst : CrossFadeState.showSecond,
        secondChild: buildWrap(context),
        firstChild: buildColumn(),
      ),
    );
  }

  Wrap buildWrap(BuildContext context) {
    return Wrap(
      children: json.map(blockMap).toList().cast<Widget>(),
    );
  }

  Widget buildColumn() {
    return Column(
      children: json.map(listMap).toList().cast<Widget>(),
    );
  }
}
