/*
 * Author: yeras-is
 *
 */

import 'package:flutter/cupertino.dart';
import 'dart:math';
import 'package:flutter/material.dart';

class SpeedDial extends StatefulWidget {
  final double size;
  final List<SDPair> iconsList;
  final Color buttonColor;
  final Color iconColor;
  final double elevation;
  final IconData openIcon;
  final IconData closeIcon;

  const SpeedDial({
    Key key,
    this.size = 45,
    this.iconsList,
    this.buttonColor,
    this.iconColor,
    this.elevation,
    this.openIcon,
    this.closeIcon,
  }) : super(key: key);

  @override
  _SpeedDialState createState() => _SpeedDialState();
}

class _SpeedDialState extends State<SpeedDial> with TickerProviderStateMixin {
  AnimationController _controller;

  @override
  void initState() {
    _controller = AnimationController(
      vsync: this,
      duration: const Duration(milliseconds: 500),
    );
    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 10),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.end,
        crossAxisAlignment: CrossAxisAlignment.end,
        children: (widget.iconsList ?? []).map(
          (e) {
            Widget child = Container(
              padding: EdgeInsets.only(top: 5, bottom: 5),
              child: ScaleTransition(
                scale: CurvedAnimation(
                  parent: _controller,
                  curve: Interval(
                      0.0, 1.0 - widget.iconsList.indexOf(e) / widget.iconsList.length / 2.0,
                      curve: Curves.easeOut),
                ),
                child: MaterialButton(
                  elevation: widget.elevation ?? 10,
                  padding: EdgeInsets.all(5),
                  color: widget.buttonColor ?? CupertinoTheme.of(context).scaffoldBackgroundColor,
                  shape: CircleBorder(),
                  child: Icon(
                    e.icon,
                    size: widget.size,
                    color: widget.iconColor ?? Theme.of(context).primaryColor,
                  ),
                  onPressed: e.onPressed,
                ),
              ),
            );
            return child;
          },
        ).toList()
          ..add(
            MaterialButton(
              elevation: widget.elevation ?? 10,
              padding: EdgeInsets.all(5),
              color: widget.buttonColor ?? CupertinoTheme.of(context).scaffoldBackgroundColor,
              shape: CircleBorder(),
              child: AnimatedBuilder(
                animation: _controller,
                builder: (BuildContext context, Widget child) {
                  return Transform(
                    transform: Matrix4.rotationZ(_controller.value * 0.5 * pi),
                    alignment: FractionalOffset.center,
                    child: Icon(
                      _controller.isDismissed
                          ? widget.openIcon ?? Icons.add
                          : widget.closeIcon ?? Icons.close,
                      size: widget.size,
                    ),
                  );
                },
              ),
              onPressed: () {
                if (_controller.isDismissed) {
                  _controller.forward();
                } else {
                  _controller.reverse();
                }
              },
            ),
          ),
      ),
    );
  }
}

class SDPair {
  final VoidCallback onPressed;
  final IconData icon;

  const SDPair({@required this.onPressed, @required this.icon});
}
